class HTTPStatus(Exception):
	__slots__ = ("_status", "_message")
	_content_type : str = ""

	def __init__(self, status : str, message : str="", content_type : str=""):
		self._status = status
		self._message = message
		if content_type:
			self._content_type = content_type
		super().__init__(self.message)

	@property
	def message(self) -> str | dict:
		if self.content_type == "application/json" and type(self._message) == str:
			return self.json_message(self._status, self._message)
		return self._message

	@property
	def content_type(self) -> str:
		return self._content_type

	@property
	def status(self) -> str:
		return self._status

	@staticmethod
	def json_message(status, message) -> dict:
		return {"status": False, "code": status, "message": message}

	def __str__(self) -> str:
		return "HTTPStatus: " + str({self._status}) + " -> " + {self._message}


class Middleware:
	__slots__ = ()

	def begin(self, req : object, resp : object) -> None:
		pass

	def routed(self, req : object, resp : callable, route_args : dict) -> None:
		pass

	def end(self, req : object, resp : object, route_args : dict) -> None:
		pass
