from sys import prefix
try:
	from orjson import loads
except ModuleNotFoundError:
	try:
		from ujson import loads
	except ModuleNotFoundError:
		from json import loads
from .route import Route
from .request import Request
from .response import Response
from .lib import HTTPStatus, Middleware


class WSGI:
	__slots__ = ("_content_type", "_middleware", "_charset")

	def __init__(self,
		middleware : Middleware=None,
		content_type : str="application/json",
		charset : str="UTF-8"
	):
		self._content_type = content_type
		self._charset = charset
		if middleware:
			if isinstance(middleware, Middleware):
				self._middleware = middleware
			else:
				raise Exception("Invalid middleware object")
		else:
			self._middleware = None
		HTTPStatus.content_type = content_type

	def __call__(self, env : object, start_response : callable) -> bytes:
		request = Request(env)
		response = Response(start_response, content_type=self._content_type, charset=self._charset)

		try:
			self._middleware and self._middleware.begin(request, response)
			method, args, not_allowed, route_args = Route.get(env["PATH_INFO"], env["REQUEST_METHOD"])
			data = None
			if not method:
				if not_allowed:
					raise HTTPStatus("405 METHOD NOT ALLOWED", "405 METHOD NOT ALLOWED")
				else:
					raise HTTPStatus("404 NOT FOUND", "404 NOT FOUND")

			self._middleware and self._middleware.routed(request, response, route_args)

			if (body := method(request, response, *args)) is not None:
				response.set(*body) if type(body) == tuple else response.set(body)
			elif not response.status:
				raise HTTPStatus("501 NOT IMPLEMENTED", "501 NOT IMPLEMENTED")

			self._middleware and self._middleware.end(request, response, route_args)
		except HTTPStatus as s:
			response.set(s.message, s.status, self._content_type)

		yield response()
